/*
 *  TeaXStyle (C) 2001 Pierpaolo Di Panfilo  <pippo_dp@libero.it>
 *
 *  All rights reserved.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the Q Public License and the BSD license.
 *
 *  This library is free software; you should have received a copy of the
 *  Q Public License and of the BSD License along with this library;
 *  see the files COPYING.QPL and COPYING.BSD.
 *
 */


#ifndef TEAXSTYLE_H
#define TEAXSTYLE_H

#include <kstyle.h>
#include <qpainter.h>
#include <qdrawutil.h>
#include <qpalette.h>
#include <qwidget.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qbitmap.h>
#include <qintdict.h>
#include <qstrlist.h>
#include <qobject.h>
#include <limits.h>
#include <ksharedpixmap.h>
#include <krootpixmap.h>

class KPixmap;
class GradientSet;
class TransMenuHandler;
class TransKickerHandler;
class KRootPixmapData;
class KickerRootPixmap;

#define MENU_CONFIG_CHANGED 667

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Enumerated Types
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

enum GradientType{ VSmallest=0, VSmall, VMed, VLarge,
                   HSmallest,   HSmall, HMed, HLarge,
                   VSmallestRev, VSmallRev, VMedRev, VLargeRev,
		   HSmallestRev, HSmallRev, HMedRev, HLargeRev
                  };

enum TransType { none=0, TransFollowBg, Custom };

enum ScrollType{ NoScroll=0, LeftRight, RightLeft, UpDown, DownUp };

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CLASS : TeaXStyle
//////////////////////////////////////////////////////////////////////////////////////////////////////////////


class TeaXStyle : public KStyle
{
friend class TransMenuHandler;
friend class TransKickerHandler;
	public:
		// Consturctor
    TeaXStyle();
		// Destructor
    ~TeaXStyle() { ; };

		// Buttons
    QBrush bgColor(void){return bgBrush;}
		virtual void drawPushButtonLabel (QPushButton *btn, QPainter *p);
    void 	drawButton(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, bool sunken = FALSE, const QBrush *fill = 0 );
    void 	drawButtonMask(QPainter *p, int x, int y, int w, int h);
    QRect buttonRect(int x, int y, int w, int h);
    void 	drawBevelButton(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, bool sunken = FALSE, const QBrush *fill = 0 );
    void 	drawPushButton(QPushButton *btn, QPainter *p);
		// Checkboxes
    QSize indicatorSize() const;
		void  drawIndicator(QPainter* p, int x, int y, int w, int h, const QColorGroup &g, int state, bool down = FALSE, bool enabled = TRUE );
    void  drawIndicatorMask(QPainter *p, int x, int y, int w, int h, int);
		// Radio buttons
    QSize exclusiveIndicatorSize() const;
    void  drawExclusiveIndicator(QPainter* p,  int x, int y, int w, int h, const QColorGroup &g, bool on,  bool down = FALSE, bool enabled =  TRUE );
    void  drawExclusiveIndicatorMask(QPainter *p, int x, int y, int w,  int h, bool);
		// Combo buttons
		void  drawComboButton(QPainter *p, int x, int y, int w, int h,  const QColorGroup &g, bool sunken = FALSE,  bool editable = FALSE, bool enabled = TRUE, const QBrush *fill = 0 );
    QRect comboButtonRect(int x, int y, int w, int h);
    QRect comboButtonFocusRect(int x, int y, int w, int h);
    void 	drawComboButtonMask(QPainter *p, int x, int y, int w, int h);
		// Scrollbars
    void 	drawScrollBarControls(QPainter*,  const QScrollBar*, int sliderStart, uint controls, uint activeControl );
    QStyle::ScrollControl scrollBarPointOver(const QScrollBar *sb,  int sliderStart, const QPoint &p);
    void scrollBarMetrics(const QScrollBar *sb, int &sliderMin, int &sliderMax, int &sliderLength, int &buttonDim);
    int  sliderLength() const;
    void drawArrow(QPainter *p, Qt::ArrowType type, bool down, int x, int y, int w, int h, const QColorGroup &g, bool enabled=true, const QBrush *fill = 0);
    void drawSlider(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, Orientation orient,  bool tickAbove, bool tickBelow);
    void drawSliderMask(QPainter *p, int x, int y, int w, int h, Orientation orient, bool, bool);
    void drawFocusRect(QPainter *p, const QRect &r, const QColorGroup &g, const QColor *pen, bool atBorder);
		// Progress bar
    void drawKProgressBlock(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, QBrush *fill);
		// Menus
    void drawPopupPanel(QPainter* p, int x, int y, int w, int h, const QColorGroup &g, int lineWidth, const QBrush *fill);
    void drawKMenuItem(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, bool active,    QMenuItem *item, QBrush *fill=NULL);
    void drawKMenuBar(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, bool macMode,  QBrush *fill=NULL);
    void drawPopupMenuItem(QPainter *p, bool checkable, int maxpmw, int tab, QMenuItem *mi, const QPalette &pal, bool act, bool enabled, int x, int y, int w, int h);
    int  popupMenuItemHeight(bool c, QMenuItem *mi, const QFontMetrics &fm);
    void polishPopupMenu(QPopupMenu *mnu);
		// Toolbars
    void drawKToolBar(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, KToolBarPos type,  QBrush *fill=NULL);
    void drawKBarHandle(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, KToolBarPos type, QBrush *fill=NULL);
    void drawKToolBarButton(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, bool sunken=false, bool raised = true, bool enabled = true, bool popup = false, KToolButtonType icontext = Icon,                        const QString& btext=QString::null, const QPixmap *icon=NULL,  QFont *font=NULL, QWidget *btn=NULL);
		// Tabs
    void drawTab(QPainter *p, const QTabBar *tabBar, QTab *tab, bool selected);
    void drawTabMask( QPainter*, const QTabBar*, QTab*, bool selected );
    void tabbarMetrics(const QTabBar *t, int &hFrame, int &vFrame, int &overlap);
		// Splitters
    void drawSplitter(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, Orientation orient);
    int  splitterWidth() const {return(6);}
		// Panels
    void drawPanel(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, bool sunken, int lineWidth, const QBrush *fill);
		// Kicker
    void drawKickerAppletHandle(QPainter *p, int x, int y, int w, int h,  const QColorGroup &g, QBrush *);
    void drawKickerTaskButton(QPainter *p, int x, int y, int w, int h, const QColorGroup &g, const QString &title, bool active, QPixmap *icon, QBrush *fill);
    // for repainting toolbuttons when the toolbar is resized
    bool eventFilter(QObject *obj, QEvent *ev);
		// misc
    int  defaultFrameWidth() const {return(2);}
    virtual void polish(QWidget *w);
    virtual void unPolish(QWidget *w);
    virtual void polish(QApplication *app);
    virtual void unPolish(QApplication *app);
    virtual void polish(QPalette &appPal);


	protected:
		// Draw gradients
    void drawVGradient(QPainter *p, const QColorGroup &g, QColorGroup::ColorRole type, int x, int y, int w, int h, int sx, int sy, int tw, int th, bool flip=false);
    void drawHGradient(QPainter *p, const QColorGroup &g, QColorGroup::ColorRole type, int x, int y, int w, int h, int sx, int sy, int tw, int th, bool flip=false);
		// Scrollbars
    void drawSBDeco(QPainter *p, const QRect &r, const QColorGroup &g, bool horiz);
    void drawSBButton(QPainter *p, const QRect &r, const QColorGroup &g, bool down=false, bool horiz = true, bool reverse = false);

	private:
    bool highcolor;
    QWidget *highlightWidget;
    bool menuAni, menuFade;
    QIntDict<GradientSet>gDict;
    TransMenuHandler *menuHandler;
    TransKickerHandler *kickerHandler;
    QBrush bgBrush, menuBrush;
    QBitmap lightBmp;
    QBitmap grayBmp;
    QBitmap dgrayBmp;
    QBitmap maskBmp;
    QBitmap xBmp;
};

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CLASS : GradientSet
//////////////////////////////////////////////////////////////////////////////////////////////////////////////

class GradientSet
{
	public:
    GradientSet(const QColor &baseColor);
    ~GradientSet();
    KPixmap* gradient(GradientType type);
    QColor* color(){return(&c);}
	private:
    KPixmap *gradients[16];
    QColor c;
};


class TransMenuHandler : public QObject
{
    Q_OBJECT
public:

    TransMenuHandler(QObject *parent);
    ~TransMenuHandler(){;}
    void loadSettings();
    int transType(){return(type);}
    KPixmap *pixmap(WId id){return(pixDict.find(id));}
    const QColor& textColor(){return(fgColor);}
    const QColor& bgColor(){return(menuColor);}
    bool useShadowText(){return(shadowText);}
    bool useBoldText(){return(boldText);}

protected slots:
    void slotKIPCMessage(int id, int);

protected:
    bool eventFilter(QObject *obj, QEvent *ev);
    void stripePixmap(KPixmap &pix);

    QColor menuColor, fgColor;
    int menuOpacity;
    int type;
    bool shadowText, boldText, menuStriped;
    QIntDict<KPixmap>pixDict;
};

class KickerRootPixmap: public QObject
{
    Q_OBJECT
public:
    KickerRootPixmap(QWidget *widget);
    ~KickerRootPixmap();
    void setFadeEffect(double strength, QColor color);
    void stripePixmap();
    void start();
    void stop();
    bool checkAvailable(bool ignored);
    void repaint(bool force);
    void loadSettings();

public slots:
    void repaint();

protected:
    virtual bool eventFilter(QObject *, QEvent *);

protected slots:
    void slotKIPCMessage(int id, int);

private slots:
    void slotBackgroundChanged(int);
    void slotDone(bool);

private:
    void enableExports();
    int currentDesktop();

    bool m_bActive, m_bInit;
    int m_Desk;

    QRect m_Rect;
    QPalette pal;
    QBrush bgBrush;
    QWidget *m_pWidget;
    QWidget *m_TopWidget;
    QTimer *m_pTimer;
    KSharedPixmap *m_pPixmap;
    QColor panelColor;
    double panelOpacity;
    int panelType;
    bool panelStripe;
};

#endif
